#include "cell.h"
#include <iostream>
using namespace std;

/// Create static variables (set to actual values in main())
double cell::D = 0.0;
double cell::dx = 0.0;
double cell::dy = 0.0;
double cell::k = 0.0;
int cell::width = 0;
int cell::height = 0;

cell::cell():
atomic()
{
}

cell::cell(const cell& src):
atomic(src),
q(src.q),
du(src.du),
x(src.x),
y(src.y)
{
	for (int i = 0; i < 4; i++)
		u[i] = src.u[i];
}

const cell& cell::operator=(const cell& src)
{
	atomic::operator=(src);
	q = src.q;
	du = src.du;
	for (int i = 0; i < 4; i++)
		u[i] = src.u[i];
	x = src.x;
	y = src.y;
	return *this;
}

void cell::set_pos(int xx, int yy)
{
	x = xx;
	y = yy;
}

void cell::set_init_conditions(double u0[5])
{
	q.setParams(D);
	for (int i = 0; i < 4; i++)
		u[i] = u0[i];
	du = compute_derivative(u0[CENTER]);
	q.init(&(u0[CENTER]),&du);
}

void cell::init()
{
	hold(q.ta());
}

void cell::delta_int()
{
	du = compute_derivative();
	q.integ(q.ta(),&du);
	hold(q.ta());
}

void cell::delta_ext(stime_t e, const adevs_bag<PortValue>& input)
{
	for (int i = 0; i < input.getSize(); i++)
	{
		if (input.get(i).value.x == x-1) u[LEFT] = input.get(i).value.s;
		else if (input.get(i).value.x == x+1) u[RIGHT] = input.get(i).value.s;
		else if (input.get(i).value.y == y-1) u[DOWN] = input.get(i).value.s;
		else if (input.get(i).value.y == y+1) u[UP] = input.get(i).value.s;
	}
	du = compute_derivative();
	q.integ(e,&du);
	hold(q.ta());
}

void cell::delta_conf(const adevs_bag<PortValue>& input) 
{
	delta_ext(q.ta(),input);
}

void cell::output_func(adevs_bag<PortValue>& o)
{
	cell_state s;
	s.x = x;
	s.y = y;
	s.s = q.out();
	output(out,s,o);
}

void cell::gc_output(adevs_bag<PortValue>& g)
{
}

void cell::report(ofstream& stream) const
{
	stream << q.state() << " ";
}

cell::~cell()
{
}

object* cell::clone() const
{
	return new cell(*this);
}

double cell::compute_derivative()
{
	return compute_derivative(q.state());
}

double cell::compute_derivative(double uc)
{
	// Set boundary conditions at the edge cells
	if (x == 0) u[LEFT] = 0.0; // uc;
	if (x == width-1) u[RIGHT] = 0.0; // uc;
	if (y == 0) u[DOWN] = 0.0; // uc;
	if (y == height-1) u[UP] = 0.0; // uc;
	// compute new time derivative using center differences in space
	double du = (k*(u[LEFT]-2.0*uc+u[RIGHT])/(dx*dx) +
				k*(u[UP]-2.0*uc+u[DOWN])/(dy*dy));
//	if (fabs(du)< k*D*2.0/(dx*dx)) du = 0.0;
	return du;
}
