#include "CDF.h"

const port_t CDF::flux_left = 10;
const port_t CDF::flux_right = 20;
const port_t CDF::soln_left = 1;
const port_t CDF::soln_right = 2;

CDF::CDF():
atomic(),
index(-1)
{}

CDF::CDF(int x, double ul, double ur):
atomic(),
ul(ul),
ur(ur),
fl(0.0),
fr(0.0),
index(x)
{}

CDF::CDF(const CDF& src):
atomic(src),
ul(src.ul),
ur(src.ur),
fl(src.fl),
fr(src.fr),
index(src.index)
{}

const CDF& CDF::operator=(const CDF& src)
{
	atomic::operator=(src);
	ul = src.ul;
	ur = src.ur;
	fl = src.fl;
	fr = src.fr;
	index = src.index;
	return *this;
}

void CDF::init()
{
	compute_flux();
	hold(0.0);
}

void CDF::delta_int()
{
	passivate();
}

void CDF::delta_ext(stime_t e, const adevs_bag<PortValue>& x)
{
	delta_conf(x);
}

void CDF::delta_conf(const adevs_bag<PortValue>& x)
{
	for (int i = 0; i < x.getSize(); i++)
	{
		if (x.get(i).port == soln_left)
			ul = x.get(i).value.get_value();
		else
			ur = x.get(i).value.get_value();
	}
	compute_flux();
	hold(0.0);
}

void CDF::output_func(adevs_bag<PortValue>& y)
{
	IOtype or(IOtype::CDF,index,fr);
	IOtype ol(IOtype::CDF,index,fl);
	output(flux_right,or,y);
	output(flux_left,ol,y);
}

void CDF::compute_flux() 
{
	// Compute the wave and speed
	double wave = ur - ul;
	double s = 1.5 * (1.0 - (ur + ul));
	if (s < 0.0)
	{
		fl = s*wave;
		fr = 0.0;
	}
	else
	{
		fl = 0.0;
		fr = s*wave;
	}
}
