#ifndef __cell_h_
#define __cell_h_
#include <adevs.h>

/**
A solution point.
*/
class cell: public atomic, public cell_interface
{
	public:
	
		/**
		Cell input is a left and right flux.  Cell output
		is the solution value.  Assumes that the solution
		grid is on the x-axis.
		*/

		/// Minimum observated time advance for internal event
		static double min_ta;
		/// Grid resolution
		static double dx;
		/// Quantum sizes
		static double D[3];

		/// Create a cell with a undefined initial conditions
		cell();
		/// Create a cell with initial value u0
		cell(adevs_vec<3> u0);
		/// Copy constructor
		cell(const cell& src);
		/// Create a copy of this cell
		object* clone() const { return new cell(*this); }
		/// Assignment operator
		const cell& operator=(const cell& src);
		void init();
		void delta_int();
		void delta_ext(stime_t e, const adevs_bag<PortValue>& x);
		void delta_conf(const adevs_bag<PortValue>& x);
		void output_func(adevs_bag<PortValue>& y);
		void gc_output(adevs_bag<PortValue>& y){}
		~cell(){}
		adevs_vec<3> getState() const;
		unsigned long getOutputCount() const;
	private:
		qoden2<3> q;
		adevs_vec<3> fl, fr;
		unsigned long output_count;
};

#endif
