#include "genr.h"
#include <math.h>

const port_t genr::out = 0;

genr::genr():atomic(){}

genr::genr(double creation_mean, double service_mean, long seed):
atomic(),
mServiceMean(service_mean),
mCreationMean(creation_mean),
mSeed(seed){}

genr::genr(const genr& src):
atomic(src),
mServiceMean(src.mServiceMean),
mCreationMean(src.mCreationMean),
mSeed(src.mSeed),
r(src.r),
service_time(src.service_time){}

const genr& genr::operator=(const genr& src)
{
	atomic::operator=(src);
	mServiceMean = src.mServiceMean;
	mCreationMean = src.mCreationMean;
	mSeed = src.mSeed;
	r = src.r;
	service_time = src.service_time;
	return *this;
}

void genr::init() 
{
	r.set_seed(mSeed);
	service_time = rint(r.exp(mServiceMean))+1.0;
	hold(r.exp(mCreationMean));
}

void genr::delta_int() 
{
	service_time = rint(r.exp(mServiceMean))+1.0;
	hold(r.exp(mCreationMean));
}

void genr::output_func(adevs_bag<PortValue>& y)
{
	output(out,new entity(service_time),y);
}

void genr::gc_output(adevs_bag<PortValue>& g)
{
	for (int i = 0; i < g.getSize(); i++)
		delete g[i].value;
}
