/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002, 2003 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#ifndef __adevs_config_h_
#define __adevs_config_h_
#include "platform_opts.h"
#include <limits.h>
#include <float.h>
#include <cmath>

// All configurable types need to have a default (no arg) constructor,
// assignment operator, and copy constructor.

/**
Define the type for model inputs and outputs.
*/
#define ADEVS_IO_TYPE object*

/**
Define the type used as model ports.  The port_t type
can always be used as an alias for ports.  The port
type must support the equality (== and !=) operators.
*/
#define ADEVS_PORT_TYPE int

/**
Define the simulation clock type.  EPSILON is used as a time granule
for determined when a time t is 'equal' to a time t +- EPSILON.
The stime_t type can always be used as an alias for time.
Time should support operators sufficient to make it an Abelian
group using the + and - operators.  It should also
support ALL of the comparison (== != <= >= > <) operators
interpreted in the usual way.
*/ 
#define ADEVS_TIME_TYPE double
#define ADEVS_ZERO 0.0
#define ADEVS_EPSILON 1E-14
#define ADEVS_INFINITY DBL_MAX

/**
Support for parallel computing.
*/
// Define to use threads
// #define USE_THREADS
// Set to appropriate thread API
#define POSIX_THREADS

/// Redefines for backwards compatibility
#ifndef IO_TYPE
	#define IO_TYPE ADEVS_IO_TYPE
#endif
#ifndef PORT_TYPE
	#define PORT_TYPE ADEVS_PORT_TYPE
#endif
#ifndef TIME_TYPE
	#define TIME_TYPE ADEVS_TIME_TYPE
#endif
#ifndef INFINITY
	#define INFINITY ADEVS_INFINITY
#endif
#ifndef EPSILON 
	#define EPSILON ADEVS_EPSILON
#endif
#ifndef ZERO
	#define ZERO ADEVS_ZERO
#endif

#endif
