/***************
adevs, a library supporting discrete event simulation
Copyright (C) 2002 James Nutaro

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Bugs, comments, and questions can be sent to nutaro@ece.arizona.edu
***************/
#include "devssim.h"

devssim::devssim(devs* model, ADEVS_TIME_TYPE t0):
model(model),sim(NULL)
{
	reset(t0);
}
	
void devssim::step(ADEVS_TIME_TYPE dt)
{
	if (dt < 0.0)
		THROW_BASE_EXCEPTION("dt < 0");
	ADEVS_TIME_TYPE t_max = model->timeCurrent()+dt;
	while (sim->timeNext() <= t_max)
	{
		sim->computeInputOutput();
		sim->computeDeltaFunc(sim->timeNext());
		sim->endOfCycle();
	}
}

void devssim::step(ADEVS_TIME_TYPE dt, const adevs_bag<PortValue>& input)
{
	if (dt < 0.0)
		THROW_BASE_EXCEPTION("dt < 0");
	ADEVS_TIME_TYPE t_max = model->timeCurrent()+dt;
	while (sim->timeNext() < t_max)
	{
		sim->computeInputOutput();
		sim->computeDeltaFunc(sim->timeNext());
		sim->endOfCycle();
	}
	sim->inject(input);
	if (t_max - timeNext() <= getAdevsTimeGranule())
		sim->computeInputOutput();
	sim->computeDeltaFunc(t_max);
	sim->endOfCycle();
}

void devssim::run(ADEVS_TIME_TYPE t_max)
{
	while (sim->timeNext() <= t_max && !halt())
	{
		sim->computeInputOutput();
		sim->computeDeltaFunc(sim->timeNext());
		sim->endOfCycle();
	}
}

ADEVS_TIME_TYPE devssim::timeCurrent()
{
	return sim->timeCurrent();
}

ADEVS_TIME_TYPE devssim::timeNext()
{
	return sim->timeNext();
}

ADEVS_TIME_TYPE devssim::timeLast()
{
	return sim->timeLast();
}

void devssim::reset(ADEVS_TIME_TYPE t0)
{
	if (sim != NULL) delete sim;
	model->setTimeLast(t0);
	model->setTimeCurrent(t0);
	model->init();
	if (model->getType() == devs::ATOMIC)
		sim = new pdevs_sim((atomic*)model);
	else
		sim = new pdevs_coord((devsn*)model);
}

bool devssim::halt()
{
	return isPassive();
}

devssim::~devssim()
{
	delete sim;
}
